-- by Northern_Strike
-- 22.07.2019

-- specialization
TrailerJointBlock = {};

function TrailerJointBlock.prerequisitesPresent(specializations)
	return SpecializationUtil.hasSpecialization(Trailer, specializations);
end;

function TrailerJointBlock.registerEventListeners(vehicleType)
	SpecializationUtil.registerEventListener(vehicleType, "onLoad", TrailerJointBlock);
	SpecializationUtil.registerEventListener(vehicleType, "onPostLoad", TrailerJointBlock);
	SpecializationUtil.registerEventListener(vehicleType, "saveToXMLFile", TrailerJointBlock);
	SpecializationUtil.registerEventListener(vehicleType, "onUpdate", TrailerJointBlock);
	SpecializationUtil.registerEventListener(vehicleType, "onWriteStream", TrailerJointBlock);
	SpecializationUtil.registerEventListener(vehicleType, "onReadStream", TrailerJointBlock);
	SpecializationUtil.registerEventListener(vehicleType, "onRegisterActionEvents", TrailerJointBlock);
end;

function TrailerJointBlock.registerFunctions(vehicleType)
	SpecializationUtil.registerFunction(vehicleType, "toggleTrailerBlockState", TrailerJointBlock["toggleTrailerBlockState"]);
end;

function TrailerJointBlock:onLoad(savegame)
	local spec = self.spec_trailerJointBlock;
	spec.isValid = false;
	spec.isBlocked = false;

	if self.spec_trailer == nil or #spec.componentJoints < 2 then
		return;
	end;

	if self.isServer then
		spec.rotLimitBackup = {};
		
		print(spec.componentJoints[1].rotLimit)
		print(tostring(spec.componentJoints[1].rotLimit[1]))
		print(tostring(spec.componentJoints[1].rotLimit[2]))
		
		if spec.componentJoints[1].rotLimit == nil or
			spec.componentJoints[1].rotLimit[2] == nil then
			spec.rotLimitBackup[1] = 0;
			spec.rotLimitBackup[2] = 0;
		else
			spec.rotLimitBackup[1] = spec.componentJoints[1].rotLimit[1];
			spec.rotLimitBackup[2] = spec.componentJoints[1].rotLimit[2];
		end;
	end;

	spec.isValid = true;
end;

function TrailerJointBlock:onPostLoad(savegame)
	local spec = self.spec_trailerJointBlock;
	if spec.isValid then
		local isBlocked = spec.isBlocked;
		if savegame ~= nil and not savegame.resetVehicles then
			isBlocked = Utils.getNoNil(getXMLBool(savegame.xmlFile, savegame.key..".trailerJointBlock#blocked"), spec.isBlocked);
		end;
		self:toggleTrailerBlockState(isBlocked, false);
	end;
end;

function TrailerJointBlock:saveToXMLFile(xmlFile, key, usedModNames)
	local spec = self.spec_trailerJointBlock;
	if spec.isValid then
		setXMLBool(xmlFile, key.."#blocked", spec.isBlocked);
	end;
end;

function TrailerJointBlock:onWriteStream(streamId, connection)
	if not connection:getIsServer() then
		local spec = self.spec_trailerJointBlock;
		if spec.isValid then
			streamWriteBool(streamId, spec.isBlocked);
		end;
	end;
end;

function TrailerJointBlock:onReadStream(streamId, connection)
	if connection:getIsServer() then
		local spec = self.spec_trailerJointBlock;
		if spec.isValid then
			spec.isBlocked = streamReadBool(streamId);
			self:toggleTrailerBlockState(spec.isBlocked, true);
		end;
	end;
end;

function TrailerJointBlock:onRegisterActionEvents(isActiveForInput)
	local spec = self.spec_trailerJointBlock;

	if self.isClient and spec.isValid then
		self:clearActionEventsTable(spec.actionEvents);

		if self:getIsActiveForInput(true) then
			local actionEventId;
			_, actionEventId = self:addActionEvent(spec.actionEvents, InputAction.TRAILER_BLOCK_FRONT_JOINT, self, TrailerJointBlock.actionEventBlock, false, true, false, true, nil);

			g_inputBinding:setActionEventTextPriority(actionEventId, GS_PRIO_NORMAL);
			g_inputBinding:setActionEventTextVisibility(actionEventId, true);
			g_inputBinding:setActionEventActive(actionEventId, false);
		end;
	end;
end;

function TrailerJointBlock.actionEventBlock(self, actionName, inputValue, callbackState, isAnalog)
	local spec = self.spec_trailerJointBlock;

	local x, y, z = getRotation(self.components[1].node);
	local x1, y1, z1 = getRotation(self.components[2].node);

	if math.abs(y - y1) < 0.05 then
		self:toggleTrailerBlockState(not spec.isBlocked, false);
	else
		g_currentMission:showBlinkingWarning(g_i18n:getText("warning_INVALID_TRAILER_POSITION"), 2000);
	end;
end;

function TrailerJointBlock:onUpdate(dt, isActiveForInput, isSelected)
	local spec = self.spec_trailerJointBlock;

	if spec.isValid then
		if self.isClient then
			local actionEvent = spec.actionEvents[InputAction.TRAILER_BLOCK_FRONT_JOINT];

			if actionEvent ~= nil and self:getIsActive() then
				local text;
				if spec.isBlocked then
					text = g_i18n:getText("action_TRAILER_UNBLOCK_FRONT_JOINT");
				else
					text = g_i18n:getText("action_TRAILER_BLOCK_FRONT_JOINT");
				end;

				g_inputBinding:setActionEventActive(actionEvent.actionEventId, true);
				g_inputBinding:setActionEventText(actionEvent.actionEventId, text);
			end;
		end;
		
		if self.isServer then
			if spec.isBlocked then
				self:setComponentJointRotLimit(self.componentJoints[1], 2, 0, 0);
			else
				self:setComponentJointRotLimit(self.componentJoints[1], 2, -spec.rotLimitBackup[2], spec.rotLimitBackup[2]);
			end;
		end;
	end;
end;

function TrailerJointBlock:toggleTrailerBlockState(isBlocked, noEventSend)
	local spec = self.spec_trailerJointBlock;

	if isBlocked ~= spec.isBlocked then
		spec.isBlocked = isBlocked;

		if noEventSend == nil or noEventSend == false then
			if g_server ~= nil then
				g_server:broadcastEvent(TrailerJointBlockEvent:new(self, isBlocked), nil, nil, self);
			else
				g_client:getServerConnection():sendEvent(TrailerJointBlockEvent:new(self, isBlocked));
			end;
		end;
	end;
end;

-- MP event
TrailerJointBlockEvent = {};
TrailerJointBlockEvent_mt = Class(TrailerJointBlockEvent, Event);
InitEventClass(TrailerJointBlockEvent, "TrailerJointBlockEvent");

function TrailerJointBlockEvent:emptyNew()
	local self = Event:new(TrailerJointBlockEvent_mt);
	return self;
end;

function TrailerJointBlockEvent:new(trailer, isBlocked)
	local self = TrailerJointBlockEvent:emptyNew();

	self.trailer = trailer;
	self.trailerIsBlocked = isBlocked;

	return self;
end;

function TrailerJointBlockEvent:readStream(streamId, connection)
	self.trailer = NetworkUtil.readNodeObject(streamId);
	self.trailerIsBlocked = streamReadBool(streamId);

	self:run(connection);
end;

function TrailerJointBlockEvent:writeStream(streamId, connection)
	NetworkUtil.writeNodeObject(streamId, self.trailer);
	streamWriteBool(streamId, self.trailerIsBlocked);
end;

function TrailerJointBlockEvent:run(connection)
	if not connection:getIsServer() then
		g_server:broadcastEvent(TrailerJointBlockEvent:new(self.trailer, self.trailerIsBlocked), nil, connection, self.trailer);
	end;

	if self.trailer ~= nil then
		self.trailer:toggleTrailerBlockState(self.trailerIsBlocked, true);
	end;
end;

-- l10n adder
local xmlFile = loadXMLFile("modDesc", g_currentModDirectory.."modDesc.xml");

local i = 0;
while true do
	local key = string.format("modDesc.l10n.text(%d)", i);

	if not hasXMLProperty(xmlFile, key) then
		break;
	end;

	local name = getXMLString(xmlFile, key.."#name");
	local text = getXMLString(xmlFile, key.."."..g_languageShort);

	if name ~= nil then
		g_i18n:setText(name, text);
	end;

	i = i + 1;
end;
